
import sys
from xml.etree import ElementTree

CTEMPLATE = \
"""
/*
 * This file has been auto-generated from the introspection data available
 * in the at-spi2-core repository. The D-Bus procol is defined in this
 * repository, which can be found at:
 *
 * http://download.gnome.org/sources/at-spi2-core/0.1/
 *
 * DO NOT EDIT.
 */

%s
"""

HTEMPLATE = \
"""
/*
 * This file has been auto-generated from the introspection data available
 * in the at-spi2-core repository. The D-Bus procol is defined in this
 * repository, which can be found at:
 *
 * http://download.gnome.org/sources/at-spi2-core/0.1/
 *
 * DO NOT EDIT.
 */

#ifndef SPI_INTROSPECTION_DATA_H_
#define SPI_INTROSPECTION_DATA_H_

%s

#endif /* SPI_INTROSPECTION_DATA_H_ */
"""

DECTEMPLATE = \
"""
extern const char *%s;
"""

DEFTEMPLATE = \
"""
const char *%s = 
%s;
"""

VERSION = "0.1.7"

def convert_name (name):
	return "spi_" + name.replace (".", "_")

def convert_contents (contents):
	contents = contents.replace ("\"", "\\\"")
	literals = ["\"%s\"" % (line) for line in contents.split ("\n")]
	return "\n".join (literals)

def main (argv):
	#Open the XML file and process includes.
	tree = ElementTree.parse ("Processed.xml")
	root = tree.getroot ()

	#Open the output files.
	cfile = open ("introspection.c", "w")
	hfile = open ("introspection.h", "w")

	ccontents = ""
	hcontents = ""

	for itf in root.findall ("node/interface"):
		#Get and convert the name of the interface.
		name = convert_name (itf.attrib["name"])

		#Create the introspection string with version information.
		itf.attrib["version"] = VERSION
		contents = convert_contents (ElementTree.tostring (itf))

		hcontents += DECTEMPLATE % (name)
		ccontents += DEFTEMPLATE % (name, contents)

	cfile.write (CTEMPLATE % (ccontents))
	hfile.write (HTEMPLATE % (hcontents))

	cfile.close ()
	hfile.close ()	
	
if __name__ == "__main__":
	sys.exit(main(sys.argv))



#!/usr/bin/python

import sys
import dbus
import dbus.service
import gobject

from dbus.mainloop.glib import DBusGMainLoop

DBusGMainLoop(set_as_default=True)

class IdleStateM (object):
        def __init__(self, bus, loop):
                self._bus = bus
                self._loop = loop
                self._func = self.setup

        def idle_handler(self):
                self._func = self._func()
                if self._func == None:
                        self._func = self.teardown
                return True

        def setup(self):
                self.obj = self._bus.get_object("org.a11y.atspi.Registry",
                                                "/org/a11y/atspi/accessible/root",
                                                introspect=False)
                self.itf = dbus.Interface(self.obj, dbus_interface="org.a11y.atspi.Accessible")
                return self.register_apps

        def register_apps(self):
                #self.itf.registerApplication(":R456", ignore_reply=True)
                #self.itf.registerApplication(":R123", ignore_reply=True)
                return self.print_applications

        def print_applications(self):
                apps = self.itf.GetChildren()
                print apps
                return self.teardown

        def teardown(self):
                self._loop.quit()

def main(argv):
        bus = dbus.SessionBus()
        obj = bus.get_object("org.a11y.Bus",
                             "/org/a11y/bus",
                             introspect=False)
        itf = dbus.Interface(obj, dbus_interface="org.a11y.Bus")
        address = itf.GetAddress()

        bus = dbus.bus.BusConnection(str(address))
        loop = gobject.MainLoop()
        stateM = IdleStateM(bus, loop)
        gobject.idle_add(stateM.idle_handler)
        loop.run()

if __name__=="__main__":
        sys.exit(main(sys.argv))
